*---------------------------------------------------------------------------------------------
*
* gcc_missing_ram_test/code.s
* 10/02/2022 steve crompton
*
* this program continuously writes: 0x0000-0xFFFF to all even addresses of a disabled/missing
* bank of ram with an immediate read back of the data on the data bus after every write cycle
* the data written/read is shown on the hex display board in the logical format: [ wwwwrrrr ]
* any read of the data bus where the data returned is not 0xFFFF triggers program termination
* following termination the data for the last write/read cycle is held on the hex display
*
* ld7[5] on - indicates the test program has started
* ld7[8] on - indicates the first pass has completed 
*
*---------------------------------------------------------------------------------------------

    .global _start

    .text

    .equ hex_display,0xAB0000

    .equ board_68681,0xAD0000
    .equ duart_base_addr,board_68681+1

    .equ duart_opr_set,14*2           	/* bit set command */
    .equ duart_opr_reset,15*2         	/* bit reset command */

    .equ ram_start,0x400000		/* missing ram - start address */
    .equ ram_size, 0x400000		/* missing ram - size */

_start:

    lea.l  duart_base_addr,%a1        	/* load the address of the duart into a1 */

    move.b #0x10,duart_opr_set(%a1)   	/* turn on ld7[5] indicating program start */

restart:
    move.l #ram_start,%a2		/* set initial address */
    clr.w %d0				/* set initial data pattern */

test_loop:
    move.w %d0,(%a2)	       	        /* write data */
    move.w (%a2),%d1			/* read data */

*   move.l %a2,hex_display+0		/* display address */
    move.w %d0,hex_display+0		/* display data written */
    move.w %d1,hex_display+2		/* display data read */

    cmp.w #0xFFFF,%d1			/* check data read was 0xFFFF */
    bne.s fail

    add.w #1,%d0			/* next data pattern */

    bne.s test_loop			/* repeat until all data patterns done */

* all data patterns done

    add.l #2,%a2			/* next address */

    cmp.l #ram_start+ram_size,%a2	/* repeat until all addresses tested */
    bne.s test_loop

* all addresses tested

pass:
    move.b #0x80,duart_opr_set(%a1)   	/* turn on ld7[8] indicating pass */

    bra.s restart			/* repeat forever or until an error occurs */

* data read error - unexpected data on the 16-bit data bus

fail:
    move.b #0x10,duart_opr_reset(%a1)   /* turn off ld7[5] indicating program end */

    move.b #228,%d7                     /* return to tutor monitor */
    trap #14

    .end

*---------------------------------------------------------------------------------------------
